@@ -65,7 +65,7 @@ namespace Banshee.Sources
         public DatabaseSource (string generic_name, string name, string id, int order) : this (generic_name, name, id, order, null)
         {
         }
-        
+
         public DatabaseSource (string generic_name, string name, string id, int order, Source parent) : base (generic_name, name, order, id)
         {
             if (parent != null) {
@@ -98,15 +98,15 @@ namespace Banshee.Sources
         }
 
         public DatabaseTrackListModel DatabaseTrackModel {
-            get { return track_model ?? track_model = (Parent as DatabaseSource ?? this).CreateTrackModelFor (this); }
+            get { return track_model ?? (track_model = (Parent as DatabaseSource ?? this).CreateTrackModelFor (this)); }
             protected set { track_model = value; }
         }
 
         private IDatabaseTrackModelCache track_cache;
         protected IDatabaseTrackModelCache TrackCache {
             get {
-                return track_cache ?? track_cache = new DatabaseTrackModelCache<DatabaseTrackInfo> (
-                    ServiceManager.DbConnection, UniqueId, DatabaseTrackModel, TrackProvider);
+                return track_cache ?? (track_cache = new DatabaseTrackModelCache<DatabaseTrackInfo> (
+                    ServiceManager.DbConnection, UniqueId, DatabaseTrackModel, TrackProvider));
             }
             set { track_cache = value; }
         }
@@ -118,7 +118,7 @@ namespace Banshee.Sources
         private void DatabaseSourceInitialize ()
         {
             InitializeTrackModel ();
-            
+
             current_filters_schema = CreateSchema<string[]> ("current_filters");
 
             DatabaseSource filter_src = Parent as DatabaseSource ?? this;
@@ -188,21 +188,25 @@ namespace Banshee.Sources
                 return true;
 
             // If it's the artist or album name, then we care, since it affects the browser
+            // FIXME this should be based on what filters (aka browsers) are active.  InternetRadio,
+            // for example, has only a Genre filter.
             if (field == Banshee.Query.BansheeQuery.ArtistField || field == Banshee.Query.BansheeQuery.AlbumField) {
                 return true;
             }
 
-            ISortableColumn sort_column = (TrackModel is DatabaseTrackListModel)
-                ? (TrackModel as DatabaseTrackListModel).SortColumn : null;
+            if (DatabaseTrackModel == null) {
+                Log.Error ("DatabaseTrackModel should not be null in DatabaseSource.NeedsReloadWhenFieldChanged");
+                return false;
+            }
 
             // If it's the field we're sorting by, then yes, we care
+            var sort_column = DatabaseTrackModel.SortColumn;
             if (sort_column != null && sort_column.Field == field) {
                 return true;
             }
 
             // Make sure the query isn't dependent on this field
-            QueryNode query = (TrackModel is DatabaseTrackListModel)
-                ? (TrackModel as DatabaseTrackListModel).Query : null;
+            QueryNode query = DatabaseTrackModel.Query;
             if (query != null) {
                 if (query != last_query) {
                     query_fields = new List<QueryField> (query.GetFields ());
@@ -255,11 +259,19 @@ namespace Banshee.Sources
         public virtual bool CanDeleteTracks {
             get { return true; }
         }
-        
+
         public virtual bool ConfirmRemoveTracks {
             get { return true; }
         }
 
+        public virtual bool CanRepeat {
+            get { return true; }
+        }
+
+        public virtual bool CanShuffle {
+            get { return true; }
+        }
+
         public override string TrackModelPath {
             get { return DBusServiceManager.MakeObjectPath (DatabaseTrackModel); }
         }
@@ -267,8 +279,8 @@ namespace Banshee.Sources
         public TrackListModel TrackModel {
             get { return DatabaseTrackModel; }
         }
-        
-        public virtual bool ShowBrowser { 
+
+        public virtual bool ShowBrowser {
             get { return true; }
         }
 
@@ -291,31 +303,27 @@ namespace Banshee.Sources
         {
             return base.AcceptsUserInputFromSource (source) && CanAddTracks;
         }
-                
+
         public override bool HasViewableTrackProperties {
             get { return true; }
         }
-        
-        public override bool HasEditableTrackProperties {
-            get { return true; }
-        }
 
 #endregion
-        
+
 #region Filters (aka Browsers)
-        
+
         private IList<IFilterListModel> available_filters;
         public IList<IFilterListModel> AvailableFilters {
-            get { return available_filters ?? available_filters = new List<IFilterListModel> (); }
+            get { return available_filters ?? (available_filters = new List<IFilterListModel> ()); }
             protected set { available_filters = value; }
         }
-        
+
         private IList<IFilterListModel> default_filters;
         public IList<IFilterListModel> DefaultFilters {
-            get { return default_filters ?? default_filters = new List<IFilterListModel> (); }
+            get { return default_filters ?? (default_filters = new List<IFilterListModel> ()); }
             protected set { default_filters = value; }
         }
-        
+
         private IList<IFilterListModel> current_filters;
         public IList<IFilterListModel> CurrentFilters {
             get {
@@ -341,7 +349,7 @@ namespace Banshee.Sources
             }
             protected set { current_filters = value; }
         }
-        
+
         public void ReplaceFilter (IFilterListModel old_filter, IFilterListModel new_filter)
         {
             int i = current_filters.IndexOf (old_filter);
@@ -350,7 +358,7 @@ namespace Banshee.Sources
                 SaveCurrentFilters ();
             }
         }
-        
+
         public void AppendFilter (IFilterListModel filter)
         {
             if (current_filters.IndexOf (filter) == -1) {
@@ -358,14 +366,14 @@ namespace Banshee.Sources
                 SaveCurrentFilters ();
             }
         }
-        
+
         public void RemoveFilter (IFilterListModel filter)
         {
             if (current_filters.Remove (filter)) {
                 SaveCurrentFilters ();
             }
         }
-        
+
         private void SaveCurrentFilters ()
         {
             Reload ();
@@ -379,13 +387,13 @@ namespace Banshee.Sources
                 }
                 current_filters_schema.Set (filters);
             }
-            
+
             EventHandler handler = FiltersChanged;
             if (handler != null) {
                 handler (this, EventArgs.Empty);
             }
         }
-        
+
         private SchemaEntry<string[]> current_filters_schema;
 
 #endregion
@@ -410,7 +418,7 @@ namespace Banshee.Sources
         public virtual bool HasDependencies {
             get { return false; }
         }
-        
+
         public void RemoveTrack (int index)
         {
             if (index != -1) {
@@ -516,9 +524,9 @@ namespace Banshee.Sources
                 AddSelectedTracks (source);
             }
         }
-        
+
 #endregion
-        
+
 #region Protected Methods
 
         protected virtual void OnTracksAdded ()
@@ -582,7 +590,7 @@ namespace Banshee.Sources
                     if (track_model.CachesJoinTableEntries) {
                         rate_track_range_command = new HyenaSqliteCommand (String.Format (@"
                             UPDATE CoreTracks SET Rating = ?, DateUpdatedStamp = ? WHERE
-                                TrackID IN (SELECT TrackID FROM {0} WHERE 
+                                TrackID IN (SELECT TrackID FROM {0} WHERE
                                     {1} IN (SELECT ItemID FROM CoreCache WHERE ModelID = {2} LIMIT ?, ?))",
                             track_model.JoinTable, track_model.JoinPrimaryKey, track_model.CacheId
                         ));
@@ -604,7 +612,7 @@ namespace Banshee.Sources
             if (!ever_reloaded)
                 Reload ();
         }
-        
+
         public override void Deactivate ()
         {
             DatabaseTrackModel.InvalidateCache (false);
@@ -627,7 +635,7 @@ namespace Banshee.Sources
         {
             Log.ErrorFormat ("AddTrackRange not implemented by {0}", this);
         }
-        
+
         protected virtual void AddTrack (DatabaseTrackInfo track)
         {
             Log.ErrorFormat ("AddTrack not implemented by {0}", this);
@@ -655,14 +663,14 @@ namespace Banshee.Sources
         protected HyenaSqliteCommand prune_command;
         protected HyenaSqliteCommand PruneCommand {
             get {
-                return prune_command ?? prune_command = new HyenaSqliteCommand (String.Format (
+                return prune_command ?? (prune_command = new HyenaSqliteCommand (String.Format (
                         @"DELETE FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT ArtistID FROM CoreTracks WHERE TrackID IN (SELECT {0}));
                           DELETE FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT AlbumID FROM CoreTracks WHERE TrackID IN (SELECT {0}));",
                         track_model.TrackIdsSql
                     ),
                     artist_model.CacheId, artist_model.CacheId, 0, artist_model.Count,
                     album_model.CacheId, album_model.CacheId, 0, album_model.Count
-                );
+                ));
             }
         }
 
